﻿//
// NTFS - Myriadbits .NET NTFS library. 
// Fast MFT reader.
// Copyright (C) 2014 Myriadbits, Jochem Bakker
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// For more information, contact me at: info@myriadbits.com
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Myriadbits.NTFS
{
	public class DataBufferReader
	{
		protected int _offset = 0;
		protected int _dataoffset = 0;
		protected DataBuffer _buffer = null;

		public DataBufferReader(DataBuffer buffer, int offset, int dataoffset)
		{	
			this._buffer = buffer;
			this._offset = offset;
			this._dataoffset = dataoffset;
		}

		//
		/// Internal get data methods
		public byte GetByte(int offset) { return this._buffer.GetByte(this._dataoffset + offset); }
		public short GetShort(int offset) { return this._buffer.GetShort(this._dataoffset + offset); }
		public ushort GetUShort(int offset) { return this._buffer.GetUShort(this._dataoffset + offset); }
		public int GetInt(int offset) { return this._buffer.GetInt(this._dataoffset + offset); }
		public uint GetUInt(int offset) { return this._buffer.GetUInt(this._dataoffset + offset); }
		public long GetLong(int offset) { return this._buffer.GetLong(this._dataoffset + offset); }
		public ulong GetULong(int offset) { return this._buffer.GetULong(this._dataoffset + offset); }
		public string GetString(int offset, int length) { return this._buffer.GetString(this._dataoffset + offset, length); }
		public byte[] GetData(int offset, int length) { return this._buffer.GetData(this._dataoffset + offset, length); }
		public DateTime GetDateTime(int offset) { return this._buffer.GetDateTime(this._dataoffset + offset);}
		
		public ulong GetVar(int offset, int length) { return this._buffer.GetVar(this._dataoffset + offset, length); }

		public override string ToString()
		{
			StringBuilder sb = new StringBuilder();
			for(int n = 0; n < 128; n++)
				sb.Append(this._buffer.GetByte(n + this._dataoffset).ToString("x") + ", ");
			return sb.ToString();
		}

		public string ToStringChars()
		{
			StringBuilder sb = new StringBuilder();
			for (int n = 0; n < 128; n++)
			{
				byte dat = this._buffer.GetByte(n + this._dataoffset);
				if (dat < 32) dat = 32;
				sb.Append(Convert.ToChar(dat));
			}
			return sb.ToString();
		}

		//public string Resident_AttributeName { get { return this._buffer.GetString(this._offset + 0x40, this.NameLength * 2); } }
	}
}
